unit PCD8544;

{
Copyright (C) 2016 - Ronald Daleske


some procedures in this unit have taken some ideas of the C sources of:

http://forum.arduino.cc/index.php?topic=271079.0
and
http://playground.arduino.cc/Code/PCD8544
and
https://learn.sparkfun.com/tutorials/graphic-lcd-hookup-guide/discuss

LcdInitialise.c


Licence
=======

 LGPLv2.1 with static linking exception (See COPYING.modifiedLGPL.txt)

}


{$mode objfpc}{$H+}

interface

uses
  Threads,
  Platform,

  Devices,
  SPI,

  GlobalConst,
  GPIO;


type
  FontChar_5x8 = Array [1..5] of Byte;


const
{$I Font_5x8.inc}






var
  X_Pos, Y_Pos : Byte;

  SPIDevice:PSPIDevice;





{
     1-VCC       -> 3.3V
     2-GND       -> GND
     3-SCE       -> GPIO8 = SPI CE0
     4-RST       -> GPIO17
     5-D/C       -> GPIO27
     6-DN(MOSI)  -> GPIO10 = MOSI
     7-SCLK      -> GPIO11 = SCLK
     8-LED       ->
}



{
3 SCE Chip select Input Active low
4 RST Reset Input Active low
5 D/C Mode select Input Select between command mode (low) and data mode (high).
6 DN(MOSI) Serial data in Input
7 SCLK Serial clock Input
8 LED LED backlight supply Input Maximum voltage supply is 3.3V.
}


procedure test;





implementation





// Reset Input Active low
procedure RESET_active;
begin
  GPIODeviceOutputSet(GPIODeviceGetDefault, GPIO_PIN_17, GPIO_LEVEL_LOW);
end;


// Reset Input Active low
procedure RESET_inactive;
begin
  GPIODeviceOutputSet(GPIODeviceGetDefault, GPIO_PIN_17, GPIO_LEVEL_HIGH);
end;




// D/C Mode select Input Select between command mode (low) and data mode (high)
procedure DC_Command;
begin
  GPIODeviceOutputSet(GPIODeviceGetDefault, GPIO_PIN_27, GPIO_LEVEL_LOW);
end;


// D/C Mode select Input Select between command mode (low) and data mode (high)
procedure DC_Data;
begin
  GPIODeviceOutputSet(GPIODeviceGetDefault, GPIO_PIN_27, GPIO_LEVEL_HIGH);
end;





// Write one Byte to LCD, MSB first
procedure LCD_Write_Byte(LCD_byte : Byte);
var
  Source_Value, Dest_Value : Byte;
  Count:LongWord;
  SourceBuffer:Pointer;
  DestBuffer:Pointer;
begin

  SourceBuffer:=@Source_Value;
  DestBuffer:=@Dest_Value;

  Source_Value:=LCD_byte;

  if SPIDeviceWriteRead(SPIDevice, SPI_CS_0, SourceBuffer, DestBuffer,
                        1, SPI_TRANSFER_NONE, Count) = ERROR_SUCCESS then begin

    // do nothing

  end;

end; { procedure LCD_Write_Byte }





// Write one Data-Byte to LCD
procedure LCD_Write_Data(LCD_data : Byte);
begin

  DC_Data;
  LCD_Write_Byte(LCD_data);

end; { procedure LCD_Write_Data }





// Write one Command-Byte to LCD
procedure LCD_Write_Command(LCD_command : Byte);
begin

  DC_Command;
  LCD_Write_Byte(LCD_command);

end; { procedure LCD_Write_Command }




// gotoXY
procedure LCD_gotoXY(pos_x, pos_y : Byte);
var
  value : Byte;
begin

  value:=$80 or pos_x;
  LCD_Write_Command(value);
  value:=$40 or pos_y;
  LCD_Write_Command(value);

end; { procedure LCD_gotoXY }






// LCD_Clear
procedure LCD_Clear;
var
  LCD_counter : Word;
begin

  // (LCD_X * LCD_Y / 8) => (84x48 / 8) => 504
  for LCD_counter:=0 to 503 do
    LCD_Write_Data($00);

end; { procedure LCD_Clear }






// LCD_Picture
procedure LCD_Picture(Picture_Array : array of byte);
var
  LCD_counter : Word;
begin
  LCD_gotoXY(0,0);

  // (LCD_X * LCD_Y / 8) => (84x48 / 8) => 504
  for LCD_counter:=0 to 503 do begin
    LCD_Write_Data(Picture_Array[LCD_counter]);
  end;

end; { procedure LCD_Picture }








// LCD_Character
procedure LCD_Character(Character : Char);
var
  LCD_counter : Word;
  temp_FontChar_5x8 : FontChar_5x8;
  Font_Byte : Byte;
begin

  if Character>=' ' then begin

    temp_FontChar_5x8:=Font_5x8[Byte(Character)];

    for LCD_counter:=1 to 5 do begin

      Font_Byte:=temp_FontChar_5x8[LCD_counter];
      LCD_Write_Data(Font_Byte);

    end; { for LCD_counter:=1 to 5 do }

    LCD_Write_Data(0);

  end; { if Character>=$20 then }


end; { procedure LCD_Character }






// LCD_String
procedure LCD_String(C_String : String);
var
  LCD_counter : Word;
begin

  for LCD_counter:=1 to Length(C_String) do begin
    LCD_Character(C_String[LCD_counter]);
  end; { for LCD_counter:=1 to 5 do }

end; { procedure LCD_String }










procedure Init_PCD8544;
begin

  X_Pos := 0;
  Y_Pos := 0;

  {
  4-RST       -> GPIO17
  5-D/C       -> GPIO27
  }

  GPIODeviceFunctionSelect(GPIODeviceGetDefault, GPIO_PIN_17, GPIO_FUNCTION_OUT);
  GPIODeviceFunctionSelect(GPIODeviceGetDefault, GPIO_PIN_27, GPIO_FUNCTION_OUT);


  SPIDevice:=PSPIDevice(DeviceFindByDescription('BCM2836 SPI0 Master'));


  if SPIDeviceStart(SPIDevice, SPI_MODE_4WIRE, 100000,
                    SPI_CLOCK_PHASE_LOW, SPI_CLOCK_POLARITY_LOW) = ERROR_SUCCESS then begin


  end; { if SPIDeviceStart }


  {
  3 SCE Chip select Input Active low
  4 RST Reset Input Active low
  5 D/C Mode select Input Select between command mode (low) and data mode (high).
  6 DN(MOSI) Serial data in Input
  7 SCLK Serial clock Input
  8 LED LED backlight supply Input Maximum voltage supply is 3.3V.
  }



  // Reset Input Active low
  RESET_inactive;

  // D/C Mode select Input Select between command mode (low) and data mode (high)
  DC_Command;





  // *** RESET ***
  // Reset Input Active low
  RESET_active;
  MicrosecondDelay(20);
  RESET_inactive;




  //Tell LCD extended commands follow
  LCD_Write_Command($21);

  //Set LCD Vop (Contrast)
  LCD_Write_Command($B1);

  //Set Temp coefficent
  LCD_Write_Command($04);

  //LCD bias mode 1:48 (try 0x13)
  LCD_Write_Command($14);

  //We must send 0x20 before modifying the display control mode
  LCD_Write_Command($20);

  //Set display control, normal mode.
  LCD_Write_Command($0C);


  LCD_gotoXY(0,0);
  LCD_Clear;


end; { procedure Init_PCD8544 }





procedure test;
begin

  LCD_gotoXY(0,0);
  LCD_String('Hello World!!!');
  LCD_gotoXY(0,1);
  LCD_String('This is a test');
  LCD_gotoXY(0,2);
  LCD_String('of ultibo core');
  LCD_gotoXY(0,3);
  LCD_String('SPI-Driver');
  LCD_gotoXY(0,4);
  LCD_String('written by Ron');
  LCD_gotoXY(0,5);
  LCD_String('on 29.06.2016');


end;





{==============================================================================}
{==============================================================================}

initialization
 Init_PCD8544;

{==============================================================================}

finalization
 {Nothing}

{==============================================================================}
{==============================================================================}

end.


